#!/usr/bin/env python
# -*- coding: utf-8 -*-
#title:		test.py
#description:	10 run test (avg result might slighty differ than the ones in the tables)
#author:	A. Ortis
#date:		20170925
#version:	0.1
#usage:		test.py	<Experiment ID>[_trunc]	
#notes:		due to the huge data, we provide only a sample of data useful to reproduce the obtained results.
#==============================================================================

#from __future__ import print_function
import sys
from sys import argv
import json
from scipy.io import loadmat
from sklearn import datasets
from sklearn.model_selection import train_test_split
from sklearn.model_selection import KFold
from sklearn import svm
from sklearn.model_selection import GridSearchCV
from sklearn.metrics import classification_report
from sklearn.svm import SVC, LinearSVC
from sklearn import preprocessing
from sklearn.metrics import accuracy_score
import h5py
import numpy as np
from utils import number_of_views, get_input_file, get_default_range, perform_cv, get_best_C
import os

import pprint

VERBOSE = True
data_dir = "./dataForSVM"
k_folds = 10
NUM_TRIALS = 10

# examples of argv[1] are:  O1, O2, O3, O3_trunc, etc.
input_data = argv[1] 
s = input_data.split('_')
experiment_id = s[0]
n_views = number_of_views(experiment_id)
n_test_views = n_views

trunc = False
trunc = (len(s)>1)

if len(argv)>2:
        C_range = [float(argv[2])]     # C value provided 
else:
	C_range = get_default_range()

out_str = "\n\nExperiment ID:\t"+ experiment_id
if trunc:
	out_str = out_str + "\t (truncated at 99%)"

fname =  get_input_file(experiment_id, trunc)
data_path = data_dir + "/" + fname	# Linux
#data_path = data_dir + "\\" + fname	# Windows

if not fname:
	print "\nInput error. Please, specify a valid Experiment ID.\n"
	print()
	sys.exit(0)

print(out_str)

if not os.path.isfile(data_path):
	print "\nInput file missing.\nTo execute the code with Experiment ID \t",experiment_id,", download the following file from the repository:\t",fname
	print "\n"
	sys.exit(0)
	
	

def perform_shuffle_train_test(NUM_TRIALS,C,X_train,X_test,y_train,y_test):
    scores_10 = []
    all_X = np.concatenate([X_train, X_test])
    all_y = np.concatenate([y_train, y_test])

    for i in range(NUM_TRIALS):

	#Shuffle the train/test features at each run
        X_train, X_test, y_train, y_test = train_test_split(all_X, all_y, test_size=0.1)


	#Fit the model 
	#clf = LinearSVC(C=C, random_state = i)
	clf = LinearSVC(C=C)
	clf.fit(X_train, y_train)
		     
        #Perform testing
	y_true, y_pred = y_test, clf.predict(X_test)

	#Compute accuracy
	acc = accuracy_score(y_true, y_pred)
       	if VERBOSE:
       	    if NUM_TRIALS == 1:
                print "C value:{0}\tAccuracy score:{1}".format(C , acc) 
            else:
                print "Run n.\t{0}\tAccuracy score:{1}".format((i+1) , acc)

        #Collect the accuracy scores
        if NUM_TRIALS == 1:
            return acc
	scores_10.append(acc)
    return scores_10

print "Running on dataset\t"+ fname + "\t n.views:\t" + str(n_views)

try:	
	dataset = loadmat(file_name = data_path, chars_as_strings = True)
	dataset = dataset['DATA']

	X_train = dataset['training_instance_matrix'][0][0]
	y_train = dataset['training_label_vector'][0][0]
	X_test = dataset['testing_instance_matrix'][0][0]
	y_test = dataset['testing_label_vector'][0][0]

except:	
	# An exception is thrown if the .mat file version is 7.5 or higher
	# Read .mat file version 7.5 or higher
	dataset_file = h5py.File(data_path,'r')
	dataset = dataset_file['DATA']

	X_train = np.array(dataset['training_instance_matrix'])
	X_train = X_train.transpose()

	y_train = np.array( dataset['training_label_vector'])
	y_train = y_train.transpose()

	X_test = np.array(dataset['testing_instance_matrix'])
	X_test = X_test.transpose()

	y_test = np.array(dataset['testing_label_vector'])
	y_test = y_test.transpose()

print "Data loading:\tOK"

n_test_images = len(y_test)
y_tr = [x[0] for x in y_train]
y_te = [x[0] for x in y_test]


y_train = y_tr*n_views
y_test =  y_te*n_views


X_train = np.array(X_train)
y_train = np.array(y_train)

means_arr = []
std_arr =   []
scores = []

# get_best_C(experiment_id, trunc)	==>>	get the best C resulted from our experiments.
# or
# perform_cv(C_range, k_folds, X_train, y_train)	==>>	perform 10-fold cross validation on a representative train dataset.
#_, best_C = perform_cv(C_range, k_folds, X_train, y_train)
best_C = get_best_C(experiment_id, trunc)
print "Best C:\t" + str(best_C)

# Perform 10 run test and avarage the results (data are re-shuffled at each run)
s_10 = perform_shuffle_train_test(NUM_TRIALS,best_C,X_train,X_test,y_train,y_test)
print setFileName		
print "10 runs Average Score: {0} STD: {1}".format(np.mean(s_10), np.std(s_10))
print "\n\n"


