#!/usr/bin/env python
#title:		createCorpus.py
#description:	This script takes all the possible words extracted from images and creates a continuous stream of text.
# 			The output represents the 'corpus' on which build text mining processings (i.e., vocabularies).
#author:	A. Ortis
#date:		20170925
#version:		0.1
#usage:		
#notes:		some word pre-processing is performed to address particular cases (e.g., jeweller's --> jeweller)
#==============================================================================


import sqlite3 as lite
import sys, json, pprint
from nltk.stem import WordNetLemmatizer
from nltk.corpus import wordnet
from utils import commuteTag

project_root	=	'/home/SentimentAnalysis/'
tokens_dir	=	'preprocessedTokens/'
db_path		=	project_root + 'flickrCrawling/flickrCrossSentiment.db'
corpus_name	=	'extractedWords.txt'

#  Debug options
DEBUG = False
D_ID = None
		
con = lite.connect(db_path)
cur = con.cursor()
cur.execute("SELECT FlickrId FROM Image")
rows = cur.fetchall()

lemmatizer = WordNetLemmatizer()
corpus_file = open(corpus_name,'w')

first_word = True
for img_idx, id in enumerate(rows):
	if DEBUG and D_ID:
		id = rows[D_ID]

	print "\n\nProcessing text from image\t" + str(id[0]) + "\t" + str(img_idx) + "/" + str(len(rows))

	fname = tokens_dir + str(id[0]) + ".json"
	f = open(fname,'r')

	obj = f.read()
	obj = json.loads(obj)
	obj = obj[0]
	if DEBUG:
		pprint.pprint(obj)
	f.close()

	google_data	=	obj['googlenet']
	mvso_data	=	obj['mvsoenglish']
	places_data	=	obj['places205']
	neuraltalk	=	obj['description']

	pool = [google_data, mvso_data, places_data, neuraltalk]
	
	for s_idx,source in enumerate(pool):
		print "\n"
		for t_idx,token in enumerate(source):
			word	=	token[0].lower()
			tag	=	token[1]

			if "'" in word:			# e.g.,  jeweller's
				word = word[:-2]


			# these if statements correct ANP tagging when the adjective is tagged as preposition
			if s_idx == 1:	#mvsoenglish
				if t_idx % 2 == 0:
					tag = 'JJ'
				else:
					tag = 'NN'
			if DEBUG:
				print tag

			if tag == 'CD':
				continue	#CD: cardinal number

			if tag in ['IN','WP','CC','DT','PRP','FW','WRB']:
				print "Ignoring tag " + tag +" for the word\t" + word + "\t(press to continue)"
				pos_tag = 'n'   # as default
			else:
				pos_tag = commuteTag(tag)
			#NB: in this script the lemmatizing is repeated considering the part of speach tag
			lemma = lemmatizer.lemmatize(word, pos_tag)

			if len(word)<4:
				print word + "("+tag+")\t\t->\t" + lemma + "("+pos_tag+")"
			else:
				print word + "("+tag+")\t->\t" + lemma + "("+pos_tag+")"


			#Add word to corpus
			if first_word:
				corpus_file.write(lemma)
				first_word = False
			else:
				corpus_file.write(" "+lemma)

corpus_file.close()
