#!/usr/bin/env python
# -*- coding: utf-8 -*-
#title:		utils.py
#description:	utils functions
#author:	A. Ortis
#date:		20170925
#version:	0.1
#usage:		import the needed functions from a python enviroment	
#notes:		
#==============================================================================

from sklearn.feature_extraction.text import CountVectorizer 

import json
from sklearn.model_selection import GridSearchCV
from sklearn.svm import LinearSVC

def perform_cv(C_range, k_folds, X_train, y_train):
    
        tuned_parameters = [{'C': C_range}]
	print "Starting k-folds Cross Validation..."
        clf = GridSearchCV(LinearSVC(), tuned_parameters, cv=k_folds,n_jobs=1, scoring = 'accuracy')    
        clf.fit(X_train, y_train)
    
        print("Best parameters set found on development set:")
        print "\n"
        print(clf.best_params_)
        best_C = clf.best_params_['C']
        print "\n"
        print("Grid scores on development set:")
        print "\n"
        means = clf.cv_results_['mean_test_score']
        stds = clf.cv_results_['std_test_score']
        for mean, std, params in zip(means, stds, clf.cv_results_['params']):
            print("%0.3f (+/-%0.3f) for %r"
                % (mean, std * 2, params))
        print "\n"
        return clf, best_C
 

def get_default_range():
	return [0.001, 0.01, 0.1, 1, 10]
	
def get_input_file(experiment_id, trunc):
	
	prefix = 'DS1_'
	if trunc:
		suffix = '_all_trunc_n.mat'
	else:
		suffix = '_all_features.mat'
	
	if experiment_id in ['K1', 'K2', 'K3', 'DK1', 'DK2', 'DK3']:
		if trunc:
			suffix = '_katsurai_trunc.mat'
		else:
			suffix = '_katsurai.mat'
	
	if experiment_id == 'K1':
		return prefix + 'v_t_s' + suffix
	elif experiment_id == 'K2':
		return prefix + 'v_t' + suffix
	elif experiment_id == 'K3':
		return prefix + 'v_s' + suffix
	elif experiment_id == 'O1':
		return prefix + 'v_t_os' + suffix
	elif experiment_id == 'O2':
		return prefix + 'v_ot_s' + suffix
	elif experiment_id == 'O3':
		return prefix + 'v_ot' + suffix
	elif experiment_id == 'O4':
		return prefix + 'v_os' + suffix
	elif experiment_id == 'O5':
		return prefix + 'v_ot_os' + suffix
	elif experiment_id == 'O6':
		return prefix + 'v_t_s_ot_os' + suffix
	elif experiment_id == 'O7':
		return prefix + 'v_t_or' + suffix
	elif experiment_id == 'O8':
		return prefix + 'v_ot_or' + suffix
	elif experiment_id == 'O9':
		return prefix + 'v_or' + suffix
	elif experiment_id == 'O10':
		return prefix + 'v_t_ot' + suffix
	elif experiment_id == 'DK1':
		return prefix + 'ds_t_s' + suffix
	elif experiment_id == 'DK2':
		return prefix + 'ds_t' + suffix
	elif experiment_id == 'DK3':
		return prefix + 'ds_s' + suffix
	elif experiment_id == 'DO1':
		return prefix + 'ds_t_os' + suffix
	elif experiment_id == 'DO2':
		return prefix + 'ds_ot_s' + suffix
	elif experiment_id == 'DO3':
		return prefix + 'ds_ot' + suffix
	elif experiment_id == 'DO4':
		return prefix + 'ds_os' + suffix
	elif experiment_id == 'DO5':
		return prefix + 'ds_ot_os' + suffix
	elif experiment_id == 'DO6':
		return prefix + 'ds_t_s_ot_os' + suffix
	elif experiment_id == 'DO7':
		return prefix + 'ds_t_or' + suffix
	elif experiment_id == 'DO8':
		return prefix + 'ds_ot_or' + suffix
	elif experiment_id == 'DO9':
		return prefix + 'ds_or' + suffix
	elif experiment_id == 'DO10':
		return prefix + 'ds_t_ot' + suffix

def get_best_C(exp_id, trunc):

	if trunc:
		if exp_id in ['O2','O4','O7','O8','DK1','DK3','DO1','DO10','DO8','DO10']:
			return 0.01
	else:
		if exp_id in ['K2','K3']:
			return 0.01
	return 0.001


def number_of_views(exp_id):
	
	if exp_id in ['K1','O1','O2','O5','O7','O8','O10','DK1','DO1','DO2','DO5','DO7','DO8','DO10',]:
		return 3
	elif exp_id in ['O6','DO6']:
		return 5
	else:
		return 2

